@@ -1,3 +1,4 @@
+# coding=utf-8
 '''
 
 TSCODE: Transition State Conformational Docker
@@ -18,22 +19,19 @@ GNU General Public License for more details.
 import os
 import warnings
 from copy import deepcopy
+
 warnings.simplefilter("ignore", UserWarning)
 
 import numpy as np
-import networkx as nx
-from rmsd import kabsch
 from cclib.io import ccread
 from numpy.linalg import LinAlgError
+from rmsd import kabsch
 
-from utils import is_sigmatropic, is_vicinal, pt
-from reactive_atoms_classes import atom_type_dict
+from tscode.errors import CCReadError
+from tscode.fast_algebra import norm_of
+from tscode.reactive_atoms_classes import atom_type_dict
+from tscode.utils import graphize, is_sigmatropic, is_vicinal, pt
 
-class CCReadError(Exception):
-    '''
-    Raised when CCRead cannot read
-    the provided filename.
-    '''
 
 def align_structures(structures:np.array, indexes=None):
     '''
@@ -45,14 +43,13 @@ def align_structures(structures:np.array, indexes=None):
 
     reference = structures[0]
     targets = structures[1:]
-    if isinstance(indexes, list):
+    if isinstance(indexes, (list, tuple)):
         indexes = np.array(indexes)
-    indexes = indexes.ravel()
 
-    indexes = slice(0,len(reference)) if indexes is None else indexes
+    indexes = slice(0,len(reference)) if indexes is None or len(indexes) == 0 else indexes.ravel()
 
     reference -= np.mean(reference[indexes], axis=0)
-    for t in range(len(targets)):
+    for t, _ in enumerate(targets):
         targets[t] -= np.mean(targets[t,indexes], axis=0)
 
     output = np.zeros(structures.shape)
@@ -67,38 +64,20 @@ def align_structures(structures:np.array, indexes=None):
         # it is actually possible for the kabsch alg not to converge
             matrix = np.eye(3)
         
-        output[t+1] = np.array([matrix @ vector for vector in target])
+        # output[t+1] = np.array([matrix @ vector for vector in target])
+        output[t+1] = (matrix @ target.T).T
 
     return output
 
-def graphize(coords, atomnos):
-    '''
-    :params coords: atomic coordinates as 3D vectors
-    :params atomnos: atomic numbers as a list
-    :return connectivity graph
-    '''
-    def d_min(e1, e2):
-        return 1.2 * (pt[e1].covalent_radius + pt[e2].covalent_radius)
-        # return 0.2 + (pt[e1].covalent_radius + pt[e2].covalent_radius)
-    # if this is somewhat prone to bugs, this might help https://cccbdb.nist.gov/calcbondcomp1x.asp
-
-    matrix = np.zeros((len(coords),len(coords)))
-    for i in range(len(coords)):
-        for j in range(i,len(coords)):
-            if np.linalg.norm(coords[i]-coords[j]) < d_min(atomnos[i], atomnos[j]):
-                matrix[i][j] = 1
-
-    return nx.from_numpy_matrix(matrix)
-
 class Hypermolecule:
     '''
     Molecule class to be used within TSCoDe.
     '''
 
     def __repr__(self):
-        return self.rootname + f' {[str(atom) for atom in self.reactive_atoms_classes_dict.values()]}, ID = {id(self)}'
+        return self.rootname + f' {[str(atom) for atom in self.reactive_atoms_classes_dict[0].values()]}, ID = {id(self)}'
 
-    def __init__(self, filename, reactive_atoms, hyper=True, debug=False):
+    def __init__(self, filename, reactive_atoms=None, hyper=True, debug=False):
         '''
         Initializing class properties: reading conformational ensemble file, aligning
         conformers to first and centering them in origin.
@@ -111,7 +90,7 @@ class Hypermolecule:
         '''
 
         if not os.path.isfile(filename):
-            if filename.endswith('.xyz'):
+            if '.' in filename:
                 raise SyntaxError((f'Molecule {filename} cannot be read. Please check your syntax.'))
 
             raise SyntaxError((f'The program is trying to read something that is not a valid molecule input ({filename}). ' +
@@ -122,7 +101,7 @@ class Hypermolecule:
         self.debug = debug
         self.hyper = hyper
 
-        if len(reactive_atoms) == 0:
+        if reactive_atoms is None:
             reactive_atoms = self._set_reactive_atoms(filename)
 
         elif len(reactive_atoms) >= 4:
@@ -168,12 +147,13 @@ class Hypermolecule:
             # sets reactive atoms properties
 
             self.atomcoords = align_structures(self.atomcoords, self.get_alignment_indexes())
-            self.sigmatropic = is_sigmatropic(self)
+            self.sigmatropic = [is_sigmatropic(self, c) for c, _ in enumerate(self.atomcoords)]
             self.sp3_sigmastar = is_vicinal(self)
 
-            for index, reactive_atom in self.reactive_atoms_classes_dict.items():   
-                reactive_atom.init(self, index, update=True)
-                # update properties into reactive_atom class
+            for c, _ in enumerate(self.atomcoords):
+                for index, reactive_atom in self.reactive_atoms_classes_dict[c].items():
+                    reactive_atom.init(self, index, update=True)
+                    # update properties into reactive_atom class
 
         self.atoms = np.array([atom for structure in self.atomcoords for atom in structure])       # single list with all atomic positions
         
@@ -216,9 +196,10 @@ class Hypermolecule:
         :param reactive atoms: int or list of ints
         :return: list of indexes
         '''
+        if len(self.reactive_indexes) == 0:
+            return None
 
         indexes = set()
-
         for atom in self.reactive_indexes:
             indexes |= set(list([(a, b) for a, b in self.graph.adjacency()][atom][1].keys()))
         if self.debug: print('DEBUG--> Alignment indexes are', list(indexes))
@@ -228,31 +209,49 @@ class Hypermolecule:
         '''
         Control the type of reactive atoms and sets the class attribute self.reactive_atoms_classes_dict
         '''
-        self.reactive_atoms_classes_dict = {}
+        self.reactive_atoms_classes_dict = {c:{} for c, _ in enumerate(self.atomcoords)}
+        
+        for c, _ in enumerate(self.atomcoords):
+            for index in self.reactive_indexes:
+                symbol = pt[self.atomnos[index]].symbol
 
-        for index in self.reactive_indexes:
-            symbol = pt[self.atomnos[index]].symbol
+                neighbors_indexes = list([(a, b) for a, b in self.graph.adjacency()][index][1].keys())
+                neighbors_indexes.remove(index)
 
-            neighbors_indexes = list([(a, b) for a, b in self.graph.adjacency()][index][1].keys())
-            neighbors_indexes.remove(index)
+                atom_type = deepcopy(atom_type_dict[symbol + str(len(neighbors_indexes))])
 
-            atom_type = deepcopy(atom_type_dict[symbol + str(len(neighbors_indexes))])
+                atom_type.init(self, index)
+                # setting the reactive_atom class type
 
-            atom_type.init(self, index)
-            # setting the reactive_atom class type
+                self.reactive_atoms_classes_dict[c][index] = atom_type
 
-            self.reactive_atoms_classes_dict[index] = atom_type
-
-            if self.debug: print(f'DEBUG--> Reactive atom {index+1} is a {symbol} atom of {atom_type} type. It is bonded to {len(neighbors_indexes)} atom(s): {atom_type.neighbors_symbols}')
-            # understanding the type of reactive atom in order to align the ensemble correctly and build the correct pseudo-orbitals
+                if self.debug:
+                    print(f'DEBUG--> Reactive atom {index+1} is a {symbol} atom of {atom_type} type. It is bonded to {len(neighbors_indexes)} atom(s): {atom_type.neighbors_symbols}')
+                # understanding the type of reactive atom in order to align the ensemble correctly and build the correct pseudo-orbitals
 
     def _scale_orbs(self, value):
         '''
         Scale each orbital dimension according to value.
         '''
-        for index, atom in self.reactive_atoms_classes_dict.items():
-            orb_dim = np.linalg.norm(atom.center[0]-atom.coord)
-            atom.init(self, index, update=True, orb_dim=orb_dim*value)
+        for c, _ in enumerate(self.atomcoords):
+            for index, atom in self.reactive_atoms_classes_dict[c].items():
+                orb_dim = norm_of(atom.center[0]-atom.coord)
+                atom.init(self, index, update=True, orb_dim=orb_dim*value)
+
+    def get_r_atoms(self, c):
+        '''
+        c: conformer number
+        '''
+        return list(self.reactive_atoms_classes_dict[c].values())
+    
+    def get_centers(self, c):
+        '''
+        c: conformer number
+        '''
+        return np.array([[v for v in atom.center] for atom in self.get_r_atoms(c)])
+
+    # def calc_positioned_conformers(self):
+    #     self.positioned_conformers = np.array([[self.rotation @ v + self.position for v in conformer] for conformer in self.atomcoords])
 
     def _compute_hypermolecule(self):
         '''
@@ -261,7 +260,7 @@ class Hypermolecule:
         self.energies = [0 for _ in self.atomcoords]
 
         self.hypermolecule_atomnos = []
-        clusters = {i:{} for i in range(len((self.atomnos)))}  # {atom_index:{cluster_number:[position,times_found]}}
+        clusters = {i:{} for i, _ in enumerate(self.atomnos)}  # {atom_index:{cluster_number:[position,times_found]}}
         for i, atom_number in enumerate(self.atomnos):
             atoms_arrangement = [conformer[i] for conformer in self.atomcoords]
             cluster_number = 0
@@ -274,16 +273,16 @@ class Hypermolecule:
                 # print(f'Atom {i} in conf {j+1} weight is {weight} - rel. E was {self.energies[j+1]}')
 
                 for cluster_number, reference in deepcopy(clusters[i]).items():
-                    if np.linalg.norm(atom - reference[0]) < radii:
+                    if norm_of(atom - reference[0]) < radii:
                         clusters[i][cluster_number][1] += weight
                     else:
                         clusters[i][max(clusters[i].keys())+1] = [atom, weight]
                         self.hypermolecule_atomnos.append(atom_number)
 
-        self.weights = [[] for _ in range(len(self.atomnos))]
+        self.weights = [[] for _ in self.atomnos]
         self.hypermolecule = []
 
-        for i in range(len(self.atomnos)):
+        for i, _ in enumerate(self.atomnos):
             for _, data in clusters[i].items():
                 self.weights[i].append(data[1])
                 self.hypermolecule.append(data[0])
@@ -314,15 +313,49 @@ class Hypermolecule:
 
         hyp_name = self.rootname + '_hypermolecule.xyz'
         with open(hyp_name, 'w') as f:
-            f.write(str(sum([len(atom.center) for atom in self.reactive_atoms_classes_dict.values()]) + len(self.atomcoords[0])))
-            f.write(f'\nTSCoDe Hypermolecule for {self.rootname} - reactive indexes {self.reactive_indexes}\n')
-            orbs =np.vstack([atom_type.center for atom_type in self.reactive_atoms_classes_dict.values()]).ravel()
-            orbs = orbs.reshape((int(len(orbs)/3), 3))
-            for i, atom in enumerate(self.atomcoords[0]):
-                f.write('%-5s %-8s %-8s %-8s\n' % (pt[self.atomnos[i]].symbol, round(atom[0], 6), round(atom[1], 6), round(atom[2], 6)))
-            for orb in orbs:
-                f.write('%-5s %-8s %-8s %-8s\n' % ('X', round(orb[0], 6), round(orb[1], 6), round(orb[2], 6)))
+            for c, _ in enumerate(self.atomcoords):
+                f.write(str(sum([len(atom.center) for atom in self.reactive_atoms_classes_dict[c].values()]) + len(self.atomcoords[0])))
+                f.write(f'\nTSCoDe Hypermolecule {c} for {self.rootname} - reactive indexes {self.reactive_indexes}\n')
+                orbs =np.vstack([atom_type.center for atom_type in self.reactive_atoms_classes_dict[c].values()]).ravel()
+                orbs = orbs.reshape((int(len(orbs)/3), 3))
+                for i, atom in enumerate(self.atomcoords[c]):
+                    f.write('%-5s %-8s %-8s %-8s\n' % (pt[self.atomnos[i]].symbol, round(atom[0], 6), round(atom[1], 6), round(atom[2], 6)))
+                for orb in orbs:
+                    f.write('%-5s %-8s %-8s %-8s\n' % ('X', round(orb[0], 6), round(orb[1], 6), round(orb[2], 6)))
 
+class Pivot:
+    '''
+    (Cyclical embed)
+    Pivot object: vector connecting two lobes of a
+    molecule, starting from v1 (first reactive atom in
+    mol.reacitve_atoms_classes_dict) and ending on v2.
+
+    For molecules involved in chelotropic reactions,
+    that is molecules that undergo a cyclical embed
+    while having only one reactive atom, pivots are
+    built on that single atom.
+    '''
+    def __init__(self, c1, c2, a1, a2, index1, index2):
+        '''
+        c: centers (orbital centers)
+        v: vectors (orbital vectors, non-normalized)
+        i: indexes (of coordinates, in mol.center)
+        '''
+        self.start = c1
+        self.end = c2
+
+        self.start_atom = a1
+        self.end_atom = a2
+
+        self.pivot = c2 - c1
+        self.meanpoint = np.mean((c1, c2), axis=0)
+        self.index = (index1, index2)
+        # the pivot starts from the index1-th
+        # center of the first reactive atom
+        # and to the index2-th center of the second
+
+    def __repr__(self):
+        return f'Pivot object - index {self.index}, norm {round(norm_of(self.pivot), 3)}, meanpoint {self.meanpoint}'
 
 if __name__ == '__main__':
     # TESTING PURPOSES
@@ -358,9 +391,9 @@ if __name__ == '__main__':
     for i in (19,):
         t = Hypermolecule(test[i][0], test[i][1])
 
-        # t.reactive_atoms_classes_dict.values()[0].init(t, t.reactive_indexes[0], update=True, orb_dim=1)
-        # t.reactive_atoms_classes_dict.values()[1].init(t, t.reactive_indexes[1], update=True, orb_dim=1.5)
+        # t.reactive_atoms_classes_dict[c].values()[0].init(t, t.reactive_indexes[0], update=True, orb_dim=1)
+        # t.reactive_atoms_classes_dict[c].values()[1].init(t, t.reactive_indexes[1], update=True, orb_dim=1.5)
         # t._update_orbs()
 
         t._compute_hypermolecule()
-        t.write_hypermolecule()
\ No newline at end of file
+        t.write_hypermolecule()
