@@ -1,3 +1,4 @@
+# coding=utf-8
 '''
 
 TSCODE: Transition State Conformational Docker
@@ -15,16 +16,13 @@ GNU General Public License for more details.
 
 '''
 
-import numpy as np
 from copy import deepcopy
-from parameters import orb_dim_dict
-from utils import (
-                   pt,
-                   neighbors,
-                   norm,
-                   rot_mat_from_pointer,
-                   vec_angle
-                )
+
+import numpy as np
+
+from tscode.fast_algebra import norm, norm_of
+from tscode.parameters import orb_dim_dict
+from tscode.utils import neighbors, pt, rot_mat_from_pointer, vec_angle
 
 
 class Single:
@@ -32,7 +30,7 @@ class Single:
     def __repr__(self):
         return 'Single Bond'
 
-    def init(self, mol, i, update=False, orb_dim=None, atomcoords_index=0) -> None:
+    def init(self, mol, i, update=False, orb_dim=None, conf=0) -> None:
         '''
         '''
         self.index = i
@@ -40,8 +38,8 @@ class Single:
         neighbors_indexes = neighbors(mol.graph, i)
 
         self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
-        self.coord = mol.atomcoords[atomcoords_index][i]
-        self.other = mol.atomcoords[atomcoords_index][neighbors_indexes][0]
+        self.coord = mol.atomcoords[conf][i]
+        self.other = mol.atomcoords[conf][neighbors_indexes][0]
 
         if not mol.sp3_sigmastar:
             self.orb_vecs = np.array([norm(self.coord - self.other)])
@@ -55,12 +53,12 @@ class Single:
                         break
             # obtain the reference partner index
 
-            partner = mol.atomcoords[atomcoords_index][parnter_index]
+            partner = mol.atomcoords[conf][parnter_index]
             pivot = norm(partner - self.coord)
 
             neighbors_of_partner = neighbors(mol.graph, parnter_index)
             neighbors_of_partner.remove(i)
-            orb_vec = norm(mol.atomcoords[atomcoords_index][neighbors_of_partner[0]] - partner)
+            orb_vec = norm(mol.atomcoords[conf][neighbors_of_partner[0]] - partner)
             orb_vec = orb_vec - orb_vec @ pivot * pivot
 
             steps = 3 # number of total orbitals
@@ -72,10 +70,10 @@ class Single:
         if update:
             if orb_dim is None:
                 key = self.symbol + ' ' + str(self)
-                try:
-                    orb_dim = orb_dim_dict[key]
-                except KeyError:
-                    orb_dim = np.linalg.norm(self.coord - self.other)
+                orb_dim = orb_dim_dict.get(key)
+
+                if orb_dim is None:
+                    orb_dim = norm_of(self.coord - self.other)
                     print(f'ATTENTION: COULD NOT SETUP REACTIVE ATOM ORBITAL FROM PARAMETERS. We have no parameters for {key}. Using the bonding distance ({round(orb_dim, 3)} A).')
 
             self.center = orb_dim * self.orb_vecs + self.coord
@@ -86,7 +84,7 @@ class Sp2:
     def __repr__(self):
         return f'sp2'
 
-    def init(self, mol, i, update=False, orb_dim=None, atomcoords_index=0) -> None:
+    def init(self, mol, i, update=False, orb_dim=None, conf=0) -> None:
         '''
         '''
         self.index = i
@@ -96,8 +94,8 @@ class Sp2:
 
 
         self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
-        self.coord = mol.atomcoords[atomcoords_index][i]
-        self.others = mol.atomcoords[atomcoords_index][neighbors_indexes]
+        self.coord = mol.atomcoords[conf][i]
+        self.others = mol.atomcoords[conf][neighbors_indexes]
 
         self.vectors = self.others - self.coord # vectors connecting reactive atom with neighbors
         self.orb_vec = norm(np.mean(np.array([np.cross(norm(self.vectors[0]), norm(self.vectors[1])),
@@ -109,9 +107,9 @@ class Sp2:
         if update:
             if orb_dim is None:
                 key = self.symbol + ' ' + str(self)
-                try:
-                    orb_dim = orb_dim_dict[key]
-                except KeyError:
+                orb_dim = orb_dim_dict.get(key)
+                
+                if orb_dim is None:
                     orb_dim = orb_dim_dict['Fallback']
                     print(f'ATTENTION: COULD NOT SETUP REACTIVE ATOM ORBITAL FROM PARAMETERS. We have no parameters for {key}. Using {orb_dim} A.')
             
@@ -125,14 +123,14 @@ class Sp3:
     def __repr__(self):
         return 'sp3'
 
-    def init(self, mol, i, update=False, orb_dim=None, atomcoords_index=0) -> None:
+    def init(self, mol, i, update=False, orb_dim=None, conf=0) -> None:
 
         self.index = i
         self.symbol = pt[mol.atomnos[i]].symbol
         neighbors_indexes = neighbors(mol.graph, i)
         self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
-        self.coord = mol.atomcoords[atomcoords_index][i]
-        self.others = mol.atomcoords[atomcoords_index][neighbors_indexes]
+        self.coord = mol.atomcoords[conf][i]
+        self.others = mol.atomcoords[conf][neighbors_indexes]
         
         if not mol.sp3_sigmastar:
 
@@ -183,11 +181,11 @@ class Sp3:
                         break
             # obtain the reference partner index
 
-            pivot = norm(mol.atomcoords[atomcoords_index][parnter_index] - self.coord)
+            pivot = norm(mol.atomcoords[conf][parnter_index] - self.coord)
 
             other_neighbors = deepcopy(neighbors_indexes)
             other_neighbors.remove(parnter_index)
-            orb_vec = norm(mol.atomcoords[atomcoords_index][other_neighbors[0]] - self.coord)
+            orb_vec = norm(mol.atomcoords[conf][other_neighbors[0]] - self.coord)
             orb_vec = orb_vec - orb_vec @ pivot * pivot
 
             steps = 3 # number of total orbitals
@@ -199,9 +197,9 @@ class Sp3:
         if update:
             if orb_dim is None:
                 key = self.symbol + ' ' + str(self)
-                try:
-                    orb_dim = orb_dim_dict[key]
-                except KeyError:
+                orb_dim = orb_dim_dict.get(key)
+                
+                if orb_dim is None:
                     orb_dim = orb_dim_dict['Fallback']
                     print(f'ATTENTION: COULD NOT SETUP REACTIVE ATOM ORBITAL FROM PARAMETERS. We have no parameters for {key}. Using {orb_dim} A.')
 
@@ -251,7 +249,7 @@ class Ether:
     def __repr__(self):
         return 'Ether'
 
-    def init(self, mol, i, update=False, orb_dim=None, atomcoords_index=0) -> None:
+    def init(self, mol, i, update=False, orb_dim=None, conf=0) -> None:
         '''
         '''
         self.index = i
@@ -261,17 +259,17 @@ class Ether:
 
 
         self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
-        self.coord = mol.atomcoords[atomcoords_index][i]
-        self.others = mol.atomcoords[atomcoords_index][neighbors_indexes]
+        self.coord = mol.atomcoords[conf][i]
+        self.others = mol.atomcoords[conf][neighbors_indexes]
 
         self.orb_vecs = self.others - self.coord # vectors connecting center to each of the two substituents
 
         if update:
             if orb_dim is None:
                 key = self.symbol + ' ' + str(self)
-                try:
-                    orb_dim = orb_dim_dict[key]
-                except KeyError:
+                orb_dim = orb_dim_dict.get(key)
+                
+                if orb_dim is None:
                     orb_dim = orb_dim_dict['Fallback']
                     print(f'ATTENTION: COULD NOT SETUP REACTIVE ATOM ORBITAL FROM PARAMETERS. We have no parameters for {key}. Using {orb_dim} A.')
 
@@ -279,7 +277,8 @@ class Ether:
 
             orb_mat = rot_mat_from_pointer(np.mean(self.orb_vecs, axis=0), 90) @ rot_mat_from_pointer(norm(np.cross(self.orb_vecs[0], self.orb_vecs[1])), 180)
 
-            self.orb_vecs = np.array([orb_mat @ v for v in self.orb_vecs])
+            # self.orb_vecs = np.array([orb_mat @ v for v in self.orb_vecs])
+            self.orb_vecs = (orb_mat @ self.orb_vecs.T).T
             
             self.center = self.orb_vecs + self.coord
             # two vectors defining the position of the two orbital lobes centers
@@ -290,7 +289,7 @@ class Ketone:
     def __repr__(self):
         return 'Ketone'
 
-    def init(self, mol, i, update=False, orb_dim=None, atomcoords_index=0) -> None:
+    def init(self, mol, i, update=False, orb_dim=None, conf=0) -> None:
         '''
         '''
         self.index = i
@@ -299,17 +298,17 @@ class Ketone:
 
 
         self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
-        self.coord = mol.atomcoords[atomcoords_index][i]
-        self.other = mol.atomcoords[atomcoords_index][neighbors_indexes][0]
+        self.coord = mol.atomcoords[conf][i]
+        self.other = mol.atomcoords[conf][neighbors_indexes][0]
 
         self.vector = self.other - self.coord # vector connecting center to substituent
 
         if update:
             if orb_dim is None:
                 key = self.symbol + ' ' + str(self)
-                try:
-                    orb_dim = orb_dim_dict[key]
-                except KeyError:
+                orb_dim = orb_dim_dict.get(key)
+                
+                if orb_dim is None:
                     orb_dim = orb_dim_dict['Fallback']
                     print(f'ATTENTION: COULD NOT SETUP REACTIVE ATOM ORBITAL FROM PARAMETERS. We have no parameters for {key}. Using {orb_dim} A.')
 
@@ -322,11 +321,11 @@ class Ketone:
             # if it is a normal ketone (or an enolate), n orbital lobes must be coplanar with
             # atoms connecting to ketone C atom, or p lobes must be placed accordingly
 
-                a1 = mol.atomcoords[atomcoords_index][neighbors_of_neighbor_indexes[0]]
-                a2 = mol.atomcoords[atomcoords_index][neighbors_of_neighbor_indexes[1]]
+                a1 = mol.atomcoords[conf][neighbors_of_neighbor_indexes[0]]
+                a2 = mol.atomcoords[conf][neighbors_of_neighbor_indexes[1]]
                 pivot = norm(np.cross(a1 - self.coord, a2 - self.coord))
 
-                if mol.sigmatropic:
+                if mol.sigmatropic[conf]:
                     # two p lobes
                     self.center = np.concatenate(([pivot*orb_dim], [-pivot*orb_dim]))
 
@@ -340,8 +339,8 @@ class Ketone:
                 ketene_sub_indexes = neighbors(mol.graph, neighbors_of_neighbor_indexes[0])
                 ketene_sub_indexes.remove(neighbors_indexes[0])
 
-                ketene_sub_coords = mol.atomcoords[atomcoords_index][ketene_sub_indexes[0]]
-                n_o_n_coords = mol.atomcoords[atomcoords_index][neighbors_of_neighbor_indexes[0]]
+                ketene_sub_coords = mol.atomcoords[conf][ketene_sub_indexes[0]]
+                n_o_n_coords = mol.atomcoords[conf][neighbors_of_neighbor_indexes[0]]
 
                 # vector connecting ketene R with C (O=C=C(R)R)
                 v = (ketene_sub_coords - n_o_n_coords)
@@ -365,7 +364,7 @@ class Imine:
     def __repr__(self):
         return 'Imine'
 
-    def init(self, mol, i, update=False, orb_dim=None, atomcoords_index=0) -> None:
+    def init(self, mol, i, update=False, orb_dim=None, conf=0) -> None:
         '''
         '''
         self.index = i
@@ -375,21 +374,21 @@ class Imine:
 
 
         self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
-        self.coord = mol.atomcoords[atomcoords_index][i]
-        self.others = mol.atomcoords[atomcoords_index][neighbors_indexes]
+        self.coord = mol.atomcoords[conf][i]
+        self.others = mol.atomcoords[conf][neighbors_indexes]
 
         self.vectors = self.others - self.coord # vector connecting center to substituent
 
         if update:
             if orb_dim is None:
                 key = self.symbol + ' ' + str(self)
-                try:
-                    orb_dim = orb_dim_dict[key]
-                except KeyError:
+                orb_dim = orb_dim_dict.get(key)
+                
+                if orb_dim is None:
                     orb_dim = orb_dim_dict['Fallback']
                     print(f'ATTENTION: COULD NOT SETUP REACTIVE ATOM ORBITAL FROM PARAMETERS. We have no parameters for {key}. Using {orb_dim} A.')
         
-            if mol.sigmatropic:
+            if mol.sigmatropic[conf]:
                 # two p lobes
                 p_lobe = norm(np.cross(self.vectors[0], self.vectors[1]))*orb_dim
                 self.orb_vecs = np.concatenate(([p_lobe], [-p_lobe]))
@@ -407,7 +406,7 @@ class Sp_or_carbene:
     def __repr__(self):
         return self.type
 
-    def init(self, mol, i, update=False, orb_dim=None, atomcoords_index=0) -> None:
+    def init(self, mol, i, update=False, orb_dim=None, conf=0) -> None:
 
         self.index = i
         self.symbol = pt[mol.atomnos[i]].symbol
@@ -415,8 +414,8 @@ class Sp_or_carbene:
         
         self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
 
-        self.coord = mol.atomcoords[atomcoords_index][i]
-        self.others = mol.atomcoords[atomcoords_index][neighbors_indexes]
+        self.coord = mol.atomcoords[conf][i]
+        self.others = mol.atomcoords[conf][neighbors_indexes]
 
         self.vectors = self.others - self.coord # vector connecting center to substituent
 
@@ -454,13 +453,13 @@ class Sp_or_carbene:
             neighbors_of_neighbors_indexes[1].remove(i)
                 
             if len(neighbors_of_neighbors_indexes[0]) == 2:
-                substituent = mol.atomcoords[atomcoords_index][neighbors_of_neighbors_indexes[0][0]]
-                ketene_atom = mol.atomcoords[atomcoords_index][neighbors_indexes[0]]
+                substituent = mol.atomcoords[conf][neighbors_of_neighbors_indexes[0][0]]
+                ketene_atom = mol.atomcoords[conf][neighbors_indexes[0]]
                 self.ketene_ref = substituent - ketene_atom
 
             elif len(neighbors_of_neighbors_indexes[1]) == 2:
-                substituent = mol.atomcoords[atomcoords_index][neighbors_of_neighbors_indexes[1][0]]
-                ketene_atom = mol.atomcoords[atomcoords_index][neighbors_indexes[1]]
+                substituent = mol.atomcoords[conf][neighbors_of_neighbors_indexes[1][0]]
+                ketene_atom = mol.atomcoords[conf][neighbors_indexes[1]]
                 self.ketene_ref = substituent - ketene_atom
 
             else:
@@ -469,9 +468,9 @@ class Sp_or_carbene:
         if update:
             if orb_dim is None:
                 key = self.symbol + ' ' + self.type
-                try:
-                    orb_dim = orb_dim_dict[key]
-                except KeyError:
+                orb_dim = orb_dim_dict.get(key)
+                
+                if orb_dim is None:
                     orb_dim = orb_dim_dict['Fallback']
                     print(f'ATTENTION: COULD NOT SETUP REACTIVE ATOM ORBITAL FROM PARAMETERS. We have no parameters for {key}. Using {orb_dim} A.')
         
@@ -489,8 +488,8 @@ class Sp_or_carbene:
                     # versor connecting reactive atom neighbors
                     
                     if self.allene:
-                        ref = (mol.atomcoords[atomcoords_index][neighbors_of_neighbors_indexes[0][0]] -
-                               mol.atomcoords[atomcoords_index][neighbors_indexes[0]])
+                        ref = (mol.atomcoords[conf][neighbors_of_neighbors_indexes[0][0]] -
+                               mol.atomcoords[conf][neighbors_indexes[0]])
                     else:
                         ref = self.ketene_ref
 
@@ -528,15 +527,15 @@ class Metal:
     def __repr__(self):
         return 'Metal'
 
-    def init(self, mol, i, update=False, orb_dim=None, atomcoords_index=0) -> None:
+    def init(self, mol, i, update=False, orb_dim=None, conf=0) -> None:
 
         self.index = i
         self.symbol = pt[mol.atomnos[i]].symbol
         neighbors_indexes = neighbors(mol.graph, i)
         
         self.neighbors_symbols = [pt[mol.atomnos[i]].symbol for i in neighbors_indexes]
-        self.coord = mol.atomcoords[atomcoords_index][i]
-        self.others = mol.atomcoords[atomcoords_index][neighbors_indexes]
+        self.coord = mol.atomcoords[conf][i]
+        self.others = mol.atomcoords[conf][neighbors_indexes]
 
         self.vectors = self.others - self.coord # vectors connecting reactive atom with neighbors
 
@@ -544,7 +543,7 @@ class Metal:
         # v1 connects first bonded atom to the metal itself
 
         neighbor_of_neighbor_index = neighbors(mol.graph, neighbors_indexes[0])[0]
-        v2 = mol.atomcoords[atomcoords_index][neighbor_of_neighbor_index] - self.coord
+        v2 = mol.atomcoords[conf][neighbor_of_neighbor_index] - self.coord
         # v2 connects first neighbor of the first neighbor to the metal itself
 
         self.orb_vec = norm(rot_mat_from_pointer(np.cross(v1, v2), 120) @ v1)
@@ -613,4 +612,4 @@ metals = (
 for metal in metals:
     for bonds in range(1,9):
         bonds = str(bonds)
-        atom_type_dict[metal+bonds] = Metal()
\ No newline at end of file
+        atom_type_dict[metal+bonds] = Metal()
