@@ -1,3 +1,4 @@
+# coding=utf-8
 '''
 
 TSCODE: Transition State Conformational Docker
@@ -18,10 +19,27 @@ GNU General Public License for more details.
 # IF YOU MANUALLY EDIT THIS FILE, BE SURE NOT TO
 # CHANGE IDENTATION/WHITESPACES/NEWLINES!
 
-OPENBABEL_OPT_BOOL = False
-# Whether to run Force Field minimization with
-# Openbabel prior to the final one.
-# (set to False if no Openbabel Python bindings are available)
+FF_OPT_BOOL = False
+# Whether to run Force Field optimization with
+# prior to the final one. Set to False if no
+# Openbabel/XTB programs and python bindings
+# are installed.
+
+FF_CALC = 'None'
+# Calculator to perform Force Field optimizations.
+# Possibilites are:
+# 'OB' : Openbabel UFF and MMFF methods
+# 'GAUSSIAN' : FF methods supported by Gaussian (UFF, MMFF)
+# 'XTB' : GFN-FF method
+
+DEFAULT_FF_LEVELS = {
+    ### DO NOT REMOVE
+    ### THESE TWO LINES
+    'GAUSSIAN':'UFF',
+    'XTB':'GFN-FF',
+    'OB':'UFF',
+}
+# Default levels used to run calculations, overridden by FFLEVEL keyword
 
 CALCULATOR = 'MOPAC'
 # Calculator used to run geometry optimization.
@@ -34,7 +52,7 @@ CALCULATOR = 'MOPAC'
 DEFAULT_LEVELS = {
     'MOPAC':'PM7',
     'ORCA':'PM3',
-    'GAUSSIAN':'PM7',
+    'GAUSSIAN':'PM6',
     'XTB':'GFN2-xTB',
 }
 # Default levels used to run calculations, overridden by LEVEL keyword
@@ -42,7 +60,7 @@ DEFAULT_LEVELS = {
 COMMANDS = {
     'MOPAC':'MOPAC2016.exe',
     'ORCA':'orca.exe',
-    'GAUSSIAN':'g16.exe',
+    'GAUSSIAN':'g09.exe',
 }
 # Command with which calculators will be called from the command line
 
@@ -51,4 +69,4 @@ PROCS = 1
 
 MEM_GB = 1
 # Memory allocated for each job (Gaussian only). If you experience problems
-# in running Gaussian calculation, try setting PROCS to 1 and MEM_GB to 1.
+# in running Gaussian calculation, try setting PROCS to 1 and MEM_GB to 0.5.
\ No newline at end of file
